import { BaseConnection, WsConnection } from "tsrpc";
import { server } from "../..";
import { RandomEx } from "../../Engine/Utils/Number/RandomEx";
import { EGameState } from "../../shared/protocols/define/enum";
import { ServiceType } from "../../shared/protocols/serviceProto";


/**
 * Room
 */
export default class Room {

	//#region static

	private static maxRoomId: number = 0;
	private static rooms: { [roomId: number]: Room } = {};

	//#endregion

	//#region public

	public RoomId: number = 0;

	//#endregion

	//#region private

	private conns: BaseConnection[] = [];

	private ready: boolean[] = [];

	private nowPlayer: [conn: BaseConnection, state: number] = undefined;

	//#endregion

	//#region Lifecycle

	/**
	 *
	 */
	constructor() {
		// 给每个新房间生成一个唯一的 ID
		this.RoomId = ++Room.maxRoomId;
		Room.rooms[this.RoomId] = this;
	}

	//#endregion

	//#region Custom

	/** GetRoom */
	public static GetRoom(roomId: number): Room {
		return this.rooms[roomId];
	}

	//#endregion

	//#region API

	/** Exit */
	public Exit(conn: BaseConnection): void {
		server.broadcastMsg("room/Exit", 0, <WsConnection<ServiceType>[]>this.conns);
	}

	/** GotoGame */
	public GotoGame(): void {
		server.broadcastMsg("room/GoToGame", 0, <WsConnection<ServiceType>[]>this.conns);
	}

	/** ChangeState */
	public ChangeState(conn: BaseConnection, state: EGameState): void {
		if (state === EGameState.Loading) {
			this.SetReady(conn);
		}
		const index: number = this.conns.indexOf(conn);
		const nowPlayer: WsConnection<ServiceType>[] = <WsConnection<ServiceType>[]>this.conns.filter((conn, i) => i === index);
		const othersPlayer: WsConnection<ServiceType>[] = <WsConnection<ServiceType>[]>this.conns.filter((conn, i) => i !== index);
		server.broadcastMsg("room/ChangeState", state + 100, nowPlayer);
		server.broadcastMsg("room/ChangeState", state + 200, othersPlayer);
	}

	//#endregion

	//#region Custom

	/** Join */
	public Join(conn: BaseConnection): void {
		this.conns.push(conn);
		this.ready.push(false);
	}

	/** SetReady */
	public SetReady(conn: BaseConnection): void {
		const index: number = this.conns.indexOf(conn);
		this.ready[index] = true;
		if (!this.ready.includes(false)) {
			const now: number = RandomEx.GetInt(0, 2);
			this.nowPlayer = [this.conns[now], EGameState.TurnCards];
			this.ChangeState(...this.nowPlayer);
		}
	}

	/** ConnCount */
	public ConnCount(): number {
		return this.conns.length;
	}

	//#endregion
}
